/*
 * minor4.h - signed minors classes
 *
 * this file is part of PJFry library
 * Copyright 2011 Valery Yundin
 */

#ifndef QUL_MINOR4_H
#define QUL_MINOR4_H

#include "minor.h"
#include "minor3.h"
#include "entry.h"
#include "cache.h"

class Minor4 : public Minor<4>
{
  friend class Golem;
  public:
    friend class SPtr<Minor4>;
    typedef SPtr<Minor4> Ptr;
    static Ptr create(const Kinem4 &k) { return Ptr(new Minor4(k)); }

    virtual Tensor::Ptr Sub(int s) { assert(0<s && s<=N); return Tensor::Ptr(mnr3[s-1]); };

    ncomplex evalD(int ep);
    ncomplex evalD(int ep, int i);
    ncomplex evalD(int ep, int i, int j);
    ncomplex evalD(int ep, int i, int j, int k);
    ncomplex evalD(int ep, int i, int j, int k, int l);

    virtual ncomplex A(int ep) { return ID0(ep); }
    virtual ncomplex A(int ep, int i) { return -IDi(ep, i); }
    virtual ncomplex A(int ep, int i, int j) { return ID2ij(ep, i, j); }
    virtual ncomplex A(int ep, int i, int j, int k) { return -ID3ijk(ep, i, j, k); }
    virtual ncomplex A(int ep, int i, int j, int k, int l) { return ID4ijkl(ep, i, j, k, l); }
    virtual ncomplex B(int ep) { return -0.5*ID1(ep); }
    virtual ncomplex B(int ep, int k) { return 0.5*ID2i(ep, k); }
    virtual ncomplex B(int ep, int k, int l) { return -0.5*ID3ij(ep, k, l); }
    virtual ncomplex C(int ep) { return 0.25*ID2(ep); }

    virtual const ncomplex* Ai(int ep) { if (not fEval[E_Di+ep]) { IDiEval(ep); } return pIDi[ep]; }
    virtual const ncomplex* Aij(int ep) { if (not fEval[E_D2ij+ep]) { ID2ijEval(ep); } return pID2ij[ep]; }
    virtual const ncomplex* Aijk(int ep) { if (not fEval[E_D3ijk+ep]) { ID3ijkEval(ep); } return pID3ijk[ep]; }
    virtual const ncomplex* Aijkl(int ep) { if (not fEval[E_D4ijkl+ep]) { ID4ijklEval(ep); } return pID4ijkl[ep]; }
    virtual const ncomplex* Bi(int ep) { if (not (ep>=1 || fEval[E_D2i+ep])) { ID2iEval(ep); } return pID2i[ep]; }
    virtual const ncomplex* Bij(int ep) { if (not (ep>=1 || fEval[E_D3ij+ep])) { ID3ijEval(ep); } return pID3ij[ep]; }

    inline
    ncomplex ID0(int ep) { return pID0[ep]; }
    ncomplex ID1(int ep) { if (pID1[ep].real() == Cache::sNAN) ID1Eval(ep); return pID1[ep]; }
    ncomplex ID2(int ep) { if (pID2[ep].real() == Cache::sNAN) ID2Eval(ep); return pID2[ep]; }
    ncomplex ID3(int ep) { if (pID3[ep].real() == Cache::sNAN) ID3Eval(ep); return pID3[ep]; }
    ncomplex ID4(int ep) { if (pID4[ep].real() == Cache::sNAN) ID4Eval(ep); return pID4[ep]; }

    ncomplex IDi(int ep, int i);
    ncomplex ID2i(int ep, int i);
    ncomplex ID3i(int ep, int i);
    ncomplex ID4i(int ep, int i);

    ncomplex ID2ij(int ep, int i, int j);
    ncomplex ID3ij(int ep, int i, int j);
    ncomplex ID4ij(int ep, int i, int j);

    ncomplex ID3ijk(int ep, int i, int j, int k);
    ncomplex ID4ijk(int ep, int i, int j, int k);

    ncomplex ID4ijkl(int ep, int i, int j, int k, int l);

#define ix(i) (i<t ? i : i-1)
    inline ncomplex I3t(int ep, int t) { return mnr3[t-1]->ID0(ep); }
    inline ncomplex I3Dt(int ep, int t) { return mnr3[t-1]->ID1(ep); }
    inline ncomplex I3D2t(int ep, int t) { return mnr3[t-1]->ID2(ep); }
    inline ncomplex I3D3t(int ep, int t) { return mnr3[t-1]->ID3(ep); }
    inline ncomplex I3D4t(int ep, int t) { return mnr3[t-1]->ID4(ep); }
    inline ncomplex I3D5t(int ep, int t) { return mnr3[t-1]->ID5(ep); }
    inline ncomplex I3D6t(int ep, int t) { return mnr3[t-1]->ID6(ep); }
    inline ncomplex I3D7t(int ep, int t) { return mnr3[t-1]->ID7(ep); }
    inline ncomplex I3Dti(int ep, int t, int i) { return t==i ? 0. : mnr3[t-1]->IDi(ep, ix(i)); }
    inline ncomplex I3D2ti(int ep, int t, int i) { return t==i ? 0. : mnr3[t-1]->ID2i(ep, ix(i)); }
    inline ncomplex I3D3ti(int ep, int t, int i) { return t==i ? 0. : mnr3[t-1]->ID3i(ep, ix(i)); }
    inline ncomplex I3D2tij(int ep, int t, int i, int j)
      { return (t==i || t==j) ? 0. : mnr3[t-1]->ID2ij(ep, ix(i), ix(j)); }
    inline ncomplex I3D3tij(int ep, int t, int i, int j)
      { return (t==i || t==j) ? 0. : mnr3[t-1]->ID3ij(ep, ix(i), ix(j)); }
    inline ncomplex I3D3tijk(int ep, int t, int i, int j, int k)
      { return (t==i || t==j || t==k) ? 0. : mnr3[t-1]->ID3ijk(ep, ix(i), ix(j), ix(k)); }
#undef ix

// pMi  - scratched lines numbering
// rrMi - matrix rank numbering
#define pM0 rrM5
#define pM1 rrM4
#define pM2 rrM3
    double M0() PURE { return pM0[0]; }
    double M1(int i, int l) PURE { return pM1[is(i,l)]; }
    double M2(int i, int j, int l, int m) PURE;
#undef pM0
#undef pM1
#undef pM2

    inline
    double maxCay() { return pmaxCay; }

  private:
    Minor4(const Kinem4 &k);

    const Kinem4 kinem;

    // flags marking evuation steps
    enum EvalM {E_None=0,
                E_M1, E_M2, E_M3,
                                                                      E_D4ijkl,
                                                              E_D4ijk=E_D4ijkl+3,
                                                       E_D4ij=E_D4ijk+3,
                                                 E_D4i=E_D4ij+3,
                                         E_D3ijk=E_D4i+3,
                                  E_D3ij=E_D3ijk+3,
                            E_D3i=E_D3ij+3,
                     E_D2ij=E_D3i+3,
               E_D2i=E_D2ij+3,
          E_Di=E_D2i+3,
    E_DUM=E_Di+3,
    E_LEN};
    std::bitset<E_LEN> fEval;

    // internally we use rank-notation, ranks 2 and 3 are defined in Minor<N>
    void evalRM4();
    static const int RM4=5;  // Binomial[DCay,DCay-R] = Binomial[5,1]
    double rrM4[RM4*(RM4+1)/2];
    double rrM5[1];

    void ID1Eval(int ep);
    void ID2Eval(int ep);
    void ID3Eval(int ep);
    void ID4Eval(int ep);

    void IDiEval(int ep);
    void ID2iEval(int ep);
    void ID3iEval(int ep);
    void ID4iEval(int ep);

    void ID2ijEval(int ep);
    void ID3ijEval(int ep);
    void ID4ijEval(int ep);

    void ID3ijkEval(int ep);
    void ID4ijkEval(int ep);

    void ID4ijklEval(int ep);

    Cache::Ival pID0;
    Cache::Ival pID1;
    Cache::Ival pID2;
    Cache::Ival pID3;
    Cache::Ival pID4;

    ncomplex pIDi[3][CIDX];                        // ir-div
    ncomplex pID2i[3][CIDX];                       // finite
    ncomplex pID3i[3][CIDX];                       // uv-div
    ncomplex pID4i[3][CIDX];                       // uv-div

    ncomplex pID2ij[3][CIDX*(CIDX+1)/2];           // ir-div symm 4x4
    ncomplex pID3ij[3][CIDX*(CIDX+1)/2];           // finite symm 4x4
    ncomplex pID4ij[3][CIDX*(CIDX+1)/2];           // uv-div symm 4x4

    ncomplex pID3ijk[3][CIDX*(CIDX+1)*(CIDX+2)/6]; // ir-div symm 4x4x4
    ncomplex pID4ijk[1][CIDX*(CIDX+1)*(CIDX+2)/6]; // finite symm 4x4x4
    ncomplex pID4ijkl[3][CIDX*(CIDX+1)*(CIDX+2)*(CIDX+3)/24];  //  ir-div symm 4x4x4x4

    double pmaxCay;

    Minor3::Ptr mnr3[N];

    bool qSmallGram;
};

typedef Entry< Kinem4, Minor4::Ptr > MEntry4;

#endif /* QUL_MINOR4_H */
